function csawh(outFile,inFile,resynth,feat_name,alpha,fs,del_width,numFilts,dLowFreq,dHighFreq,fileType,addDither)
%csawh(outFile,inFile,resynth,feat_name,alpha,fs,del_width,numFilts,dLowFreq,dHighFreq,fileType)

%%%%%%%%%%%%
% program
% parameters
%fs        = 16000;
%alpha     = 0.85; % <-- This parameter can be tuned for optimal performance, 0.85 works well in Sphinx
pow_expon = 1/15;
T_wind    = 0.0512;
T_shift   = 0.0128;
iFFTSize  = 1024;

%%%%%%%%%%%%
% load pre-computed power signal histograms
% and Gammatone filter bank
load avgHistMaxMin_SPNCC_NoCMN_sri-wsj_new.mat;
%load avgHistMaxMin_SPNCC_NoCMN_RM1-1600_new.mat;
load GammatoneFilterBank.mat;

%%%%%%%%%%%%
% read input
% waveform
fid = fopen(inFile, 'rb');
if strcmp(fileType,'wav') == 1 % <-- MS-WAV format
    fseek(fid, 44, 'bof');
else % <-- else assumed to be SPHERE format by default
    fseek(fid, 1024, 'bof'); 
end
ad_x  = fread(fid, 'int16');
fclose(fid);
ad_x = ad_x/32768;

%%%%%%%%%%%%
% resample to 16kHz 
% sampling rate
if fs ~= 16000
	ad_x = resample(ad_x,16000,fs); fs_old = fs; fs = 16000;
end

%%%%%%%%%%%%
% STFT
% parameters
N1 = floor(T_wind*fs);
N2 = floor(T_shift*fs);
W  = hamming(N1);

%%%%%%%%%%%%
% pad signal for minimum 
% of one frame length (or trim for max. of one hour)
if length(ad_x) < N1
	if size(ad_x,2) == 1
		ad_x = [ad_x;zeros(N1-length(ad_x),1)];
	else
		ad_x = [ad_x,zeros(1,N1-length(ad_x))];
	end
elseif length(ad_x) > 3600*fs
	ad_x = ad_x(1:3600*fs);
end

%%%%%%%%%%%%
% add dither to avoid
% artificial zeros (from UTD CRSS)
if addDither == 1
	dither = rand(length(ad_x), 1) + rand(length(ad_x), 1) - 1;
	spow   = std(ad_x);
	ad_x   = ad_x + 1e-6 * spow * dither;
end

%%%%%%%%%%%%
% conventional
% pre-emphasis
ad_x = filter([1 -0.97], 1, ad_x);

%%%%%%%%%%%%
% STFT
%
iNumWindows = 1 + floor((length(ad_x) - N1)/N2);
X           = zeros((iFFTSize/2) + 1, iNumWindows);
Xp          = X;

for i_FI=1:iNumWindows
    x      = ad_x( 1 + (i_FI-1)*N2 : N1 + (i_FI-1)*N2 ).* W;
    Xframe = fft(x,iFFTSize); 
    Xframe = Xframe(1:(iFFTSize/2) + 1);
    
    Xp(:,i_FI) = Xframe;
    X(:,i_FI)  = abs(Xframe);
end

%%%%%%%%%%%%
% compute filter 
% bank output
P = (transpose(H).^2)*(X.^2);

%%%%%%%%%%%%
% peak power 
% normalization
adSumPower = sum(P,2);
adSorted   = sort(adSumPower);
dMaxPower  = adSorted(round(0.98 * length(adSumPower)));
U          = (P / dMaxPower) * 1e10;

%%%%%%%%%%%%
% power-law
% nonlinearity
Pnorm = U.^pow_expon;

%%%%%%%%%%%%
% histogram-based
% processing
Pproc = histMfeat(Pnorm,pow_expon,alpha,hgram);

%%%%%%%%%%%%
% power spectrum
% weights
w = Pproc./P;
%w = sqrt(Pproc./P);

%%%%%%%%%%%%
% weight
% smoothing
[iMax,jMax] = size(w);
wS          = zeros(iMax,jMax);
M = 1; %frequency smoothing (half - 1/2) range
N = 1; %time smoothing (half - 1/2) range
for i=1:iMax
    for j=1:jMax
            wS(i,j) = mean(mean(w( max(1,i-M):min(i+M,iMax),...
                max(1,j-N):min(j+N,jMax) )));
    end
end

%%%%%%%%%%%%
% spectrogram
% reshaping
Hs = sum(H,2); Hs(Hs==0)=eps;
u  = H*wS;
u  = u./repmat(Hs,1,size(u,2));
Y  = u.*Xp;

%%%%%%%%%%%%
% reconstruct
% using OLA
y   = zeros(size(ad_x));
x_t = ifft([Y;conj(flipud(Y(2:end-1,:)))]); %to time-domain
for i_FI = 1:iNumWindows
    x = x_t(1:N1,i_FI);
    y(1 + (i_FI-1)*N2 : N1 + (i_FI-1)*N2) =...
        y(1 + (i_FI-1)*N2 : N1 + (i_FI-1)*N2) + (x .* W);
end

%%%%%%%%%%%%
% post
% de-emphasis
%y = filter(1, [1 -0.97], y);

%%%%%%%%%%%%
% resample to specified
% sampling rate
if exist('fs_old','var')
	y = resample(y,fs_old,fs); fs = fs_old;
end

%%%%%%%%%%%%
% feature extraction OR
% resynthesis
y = 0.9*(y / max(abs(y)));
if resynth == 1
    wavwrite(y,fs,16,outFile);
else
    %%%%%%%%%%%% <------- feature extraction (SPNCC) -------> %%%%%%%%%%%%
    ad_x = y;
    clearvars -except ad_x outFile x_orig fs feat_name del_width numFilts dLowFreq dHighFreq;
    %%%%%%%%%%%%
    % program
    % parameters
    %fs        = 16000;
    pow_expon = 1/15;
    T_wind    = 0.0256;
    T_shift   = 0.01;
    iFFTSize  = 1024;

    %%%%%%%%%%%%
    % conventional
    % pre-emphasis
    ad_x = filter([1 -0.97], 1, ad_x);

    %%%%%%%%%%%%
    % STFT
    %
    N1 = floor(T_wind*fs);
    N2 = floor(T_shift*fs);
    W  = hamming(N1);

    iNumWindows = 1 + floor((length(ad_x) - N1)/N2);
    X           = zeros((iFFTSize/2) + 1, iNumWindows);
    Xp          = X;

    for i_FI=1:iNumWindows
        x      = ad_x( 1 + (i_FI-1)*N2 : N1 + (i_FI-1)*N2 ).* W;
        Xframe = fft(x,iFFTSize); 
        Xframe = Xframe(1:(iFFTSize/2) + 1);

        Xp(:,i_FI) = Xframe;
        X(:,i_FI)  = abs(Xframe);
    end

    %%%%%%%%%%%%
    % compute filter
    % bank
    H = ComputeFilterResponse(numFilts,iFFTSize,dLowFreq,dHighFreq,fs);
    H = NormalizeFilterGain(abs(H), fs);

    %%%%%%%%%%%%
    % compute filter 
    % bank output
    P = (transpose(H).^2)*(X.^2);

    %%%%%%%%%%%%
    % peak power 
    % normalization
    adSumPower = sum(P,2);
    adSorted   = sort(adSumPower);
    dMaxPower  = adSorted(round(0.98 * length(adSumPower)));
    U          = (P / dMaxPower) * 1e10;

    %%%%%%%%%%%%
    % power-law
    % nonlinearity
    Pnorm = U.^pow_expon;

    %%%%%%%%%%%%
    % DCT
    %
    C  = dct(Pnorm);

    %%%%%%%%%%%%
    % CMN, delta &
    % double-delta computation
    C  = C(1:13,:);
    C  = C - repmat(mean(C,2),1,size(C,2));
    d1 = deltas(C,del_width);
    d2 = deltas(d1,del_width);

    %%%%%%%%%%%%
    % write the feature
    % in SRI-sphere format
    writesri([C;d1;d2],feat_name,outFile);
    %writefeat(outFile,[C;d1;d2]);
end
